//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/MaterialModel.h
//! @brief     Defines class MaterialModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_MATERIALMODEL_H
#define BORNAGAIN_GUI_MODELS_MATERIALMODEL_H

#include "GUI/Models/SessionModel.h"

class MaterialItem;

class MaterialModel : public SessionModel {
    Q_OBJECT

public:
    explicit MaterialModel(QObject* parent = nullptr);
    ~MaterialModel();

    MaterialModel* createCopy(SessionItem* parent = nullptr);

    MaterialItem* addRefractiveMaterial(const QString& name, double delta, double beta);
    MaterialItem* addSLDMaterial(const QString& name, double sld, double abs_term);

    MaterialItem* materialFromName(const QString& name);
    MaterialItem* materialFromIdentifier(const QString& identifier);

    //! Returns clone of given material. Clone will get the same material identifier!
    MaterialItem* cloneMaterial(MaterialItem* material);

    QVector<MaterialItem*> materialItems() const;

    void removeMaterial(const QString& identifier);
    void removeMaterial(MaterialItem* materialItem);

    //! Compares for complete equality (same material identifiers, same order of materials,...)
    bool operator==(const MaterialModel& other) const;
    bool operator!=(const MaterialModel& other) const;
};

#endif // BORNAGAIN_GUI_MODELS_MATERIALMODEL_H
