//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/LayerEditorUtils.h
//! @brief     Defines class LayerEditorUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H
#define BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H

#include "GUI/Models/DoubleDescriptor.h"

#include <QColor>
#include <QList>
#include <functional>

class QLabel;
class DoubleSpinBox;
class QGridLayout;
class SampleEditorController;
class VectorDescriptor;
class QString;
class LayerZeroRoughnessItem;
class LayerBasicRoughnessItem;
class FTDecayFunction1DItem;
class FormFactorItem;
class FTDistribution1DItem;
class RotationItem;
class QWidget;
class ItemWithParticles;
class QPushButton;

//! Utility functions to support layer oriented sample editor
class LayerEditorUtils {
public:
    static void updateLabelUnit(QLabel* label);
    static void updateLabelUnit(QLabel* label, DoubleSpinBox* editor);

    static void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                                       const DoubleDescriptors& valueDescriptors,
                                       SampleEditorController* m_ec, bool vertically,
                                       bool addSpacer);

    static void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                                       const DoubleDescriptors& valueDescriptors,
                                       SampleEditorController* m_ec, bool addSpacer);

    static void addVectorToGrid(QGridLayout* m_gridLayout, int firstCol, const VectorDescriptor& v,
                                SampleEditorController* m_ec, bool vertically, bool addSpacer);

    static QLabel* createBoldLabel(const QString& text);

    static DoubleDescriptors doubleDescriptorsOfItem(
        const std::variant<LayerZeroRoughnessItem*, LayerBasicRoughnessItem*>& item);

    static DoubleDescriptors doubleDescriptorsOfItem(FTDecayFunction1DItem* item);
    static DoubleDescriptors doubleDescriptorsOfItem(FormFactorItem* item);
    static DoubleDescriptors doubleDescriptorsOfItem(FTDistribution1DItem* item);
    static DoubleDescriptors doubleDescriptorsOfItem(RotationItem* item);

    static QWidget* createWidgetForItemWithParticles(QWidget* parentWidget,
                                                     ItemWithParticles* itemWithParticles,
                                                     SampleEditorController* ec,
                                                     bool allowRemove = true);

    static QPushButton* createAddParticleButton(QWidget* parentWidget,
                                                std::function<void(const QString&)> slot);

    static QList<QColor> predefinedLayerColors();
};

#endif // BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H
